@@ -31,9 +31,9 @@ import cgi
 import re
 from copy import deepcopy
 import warnings
-import time
 import six
 import requests
+import codecs
 
 """
 Utility functions and classes
@@ -130,24 +130,21 @@ class ResponseWrapper(object):
         return self._response.headers
 
     def read(self):
-        if not self._response.encoding:
-            return self._response.content           # bytes
-
-        return self._response.text.encode('utf-8')  # str
+        return self._response.content
 
     def geturl(self):
         return self._response.url
 
     # @TODO: __getattribute__ for poking at response
 
-def openURL(url_base, data=None, method='Get', cookies=None, username=None, password=None, timeout=30):
+def openURL(url_base, data=None, method='Get', cookies=None, username=None, password=None, timeout=30, headers=None):
     """
     Function to open URLs.
 
     Uses requests library but with additional checks for OGC service exceptions and url formatting.
     Also handles cookies and simple user password authentication.
     """
-    headers = None
+    headers = headers if headers is not None else {}
     rkwargs = {}
 
     rkwargs['timeout'] = timeout
@@ -165,7 +162,7 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
     if method.lower() == 'post':
         try:
             xml = etree.fromstring(data)
-            headers = {'Content-Type': 'text/xml'}
+            headers['Content-Type'] = 'text/xml'
         except (ParseError, UnicodeEncodeError):
             pass
 
@@ -188,18 +185,28 @@ def openURL(url_base, data=None, method='Get', cookies=None, username=None, pass
     if req.status_code in [400, 401]:
         raise ServiceException(req.text)
 
-    if req.status_code in [404]:    # add more if needed
+    if req.status_code in [404, 500, 502, 503, 504]:    # add more if needed
         req.raise_for_status()
 
     # check for service exceptions without the http header set
-    if 'Content-Type' in req.headers and req.headers['Content-Type'] in ['text/xml', 'application/xml']:
+    if 'Content-Type' in req.headers and req.headers['Content-Type'] in ['text/xml', 'application/xml', 'application/vnd.ogc.se_xml']:
         #just in case 400 headers were not set, going to have to read the xml to see if it's an exception report.
         se_tree = etree.fromstring(req.content)
-        serviceException=se_tree.find('{http://www.opengis.net/ows}Exception')
-        if serviceException is None:
-            serviceException=se_tree.find('ServiceException')
-        if serviceException is not None:
-            raise ServiceException(str(serviceException.text).strip())
+
+        # to handle the variety of namespaces and terms across services
+        # and versions, especially for "legacy" responses like WMS 1.3.0
+        possible_errors = [
+            '{http://www.opengis.net/ows}Exception',
+            '{http://www.opengis.net/ows/1.1}Exception',
+            '{http://www.opengis.net/ogc}ServiceException',
+            'ServiceException'
+        ]
+
+        for possible_error in possible_errors:
+            serviceException = se_tree.find(possible_error)
+            if serviceException is not None:
+                # and we need to deal with some message nesting
+                raise ServiceException('\n'.join([str(t).strip() for t in serviceException.itertext() if str(t).strip()]))
 
     return ResponseWrapper(req)
 
@@ -379,10 +386,7 @@ def http_post(url=None, request=None, lang='en-US', timeout=10, username=None, p
         rkwargs['auth'] = (username, password)
 
     up = requests.post(url, request, headers=headers, **rkwargs)
-    if not up.encoding:
-        return up.content           # bytes
-
-    return up.text.encode('utf-8')  # str
+    return up.content
 
 def element_to_string(element, encoding=None, xml_declaration=False):
     """
@@ -489,16 +493,16 @@ def dump(obj, prefix=''):
 
     print("%s %s.%s : %s" % (prefix, obj.__module__, obj.__class__.__name__, obj.__dict__))
 
-def getTypedValue(type, value):
-    ''' Utility function to cast a string value to the appropriate XSD type. '''
-    
-    if type=='boolean':
-       return bool(value)
-    elif type=='integer':
-       return int(value)
-    elif type=='float':
+def getTypedValue(data_type, value):
+    '''Utility function to cast a string value to the appropriate XSD type. '''
+
+    if data_type == 'boolean':
+        return bool(value)
+    elif data_type == 'integer':
+        return int(value)
+    elif data_type == 'float':
         return float(value)
-    elif type=='string':
+    elif data_type == 'string':
         return str(value)
     else:
         return value # no type casting
@@ -537,10 +541,36 @@ def extract_xml_list(elements):
 Some people don't have seperate tags for their keywords and seperate them with
 a newline. This will extract out all of the keywords correctly.
 """
-    keywords = [re.split(r'[\n\r]+',f.text) for f in elements if f.text]
-    flattened = [item.strip() for sublist in keywords for item in sublist]
-    remove_blank = [_f for _f in flattened if _f]
-    return remove_blank
+    if elements:
+        keywords = [re.split(r'[\n\r]+',f.text) for f in elements if f.text]
+        flattened = [item.strip() for sublist in keywords for item in sublist]
+        remove_blank = [_f for _f in flattened if _f]
+        return remove_blank
+    else:
+        return []
+
+
+def strip_bom(raw_text):
+    """ return the raw (assumed) xml response without the BOM
+    """
+    boms = [
+        codecs.BOM,
+        codecs.BOM_BE,
+        codecs.BOM_LE,
+        codecs.BOM_UTF8,
+        codecs.BOM_UTF16,
+        codecs.BOM_UTF16_LE,
+        codecs.BOM_UTF16_BE,
+        codecs.BOM_UTF32,
+        codecs.BOM_UTF32_LE,
+        codecs.BOM_UTF32_BE
+    ]
+
+    if not isinstance(raw_text, str):
+        for bom in boms:
+            if raw_text.startswith(bom):
+                return raw_text.replace(bom, '')
+    return raw_text
 
 
 def bind_url(url):
@@ -594,5 +624,40 @@ def which_etree():
         which_etree = 'xml.etree'
     elif 'elementree' in etree.__file__:
         which_etree = 'elementtree.ElementTree'
-    
+
     return which_etree
+
+def findall(root, xpath, attribute_name=None, attribute_value=None):
+    """Find elements recursively from given root element based on
+    xpath and possibly given attribute
+
+    :param root: Element root element where to start search
+    :param xpath: xpath defintion, like {http://foo/bar/namespace}ElementName
+    :param attribute_name: name of possible attribute of given element
+    :param attribute_value: value of the attribute
+    :return: list of elements or None
+    """
+
+    found_elements = []
+
+
+    # python 2.6 < does not support complicated XPATH expressions used lower
+    if (2, 6) == sys.version_info[0:2] and which_etree() != 'lxml.etree':
+
+        elements = root.getiterator(xpath)
+
+        if attribute_name is not None and attribute_value is not None:
+            for element in elements:
+                if element.attrib.get(attribute_name) == attribute_value:
+                    found_elements.append(element)
+        else:
+            found_elements = elements
+    # python at least 2.7 and/or lxml can do things much simplier
+    else:
+        if attribute_name is not None and attribute_value is not None:
+            xpath = '%s[@%s="%s"]' % (xpath, attribute_name, attribute_value)
+        found_elements = root.findall('.//' + xpath)
+
+    if found_elements == []:
+        found_elements = None
+    return found_elements
